﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace HKFmonitor
{
    public partial class frmDiagnostics : Form
    {
        Configuration conf;

        public string oNet = "NE";
        public string oHKF = "NE";
        public string oHGW = "NE";
        public string oPCGW = "NE";

        public frmDiagnostics(Configuration conf)
        {
            InitializeComponent();
            this.conf = conf;
            pnlWorking.Visible = false;
            btSendDiag.Enabled = false;
            btFAQ.Visible = false;

            clearAll();
        }

        private void clearAll()
        {
            lbSpeed.Text = "";
            btSendDiag.Enabled = false;
            btFAQ.Visible = false;

            pbNetT.Visible = false;
            pbNetF.Visible = false;
            pbBPt.Visible = false;
            pbBPf.Visible = false;
            pbBHt.Visible = false;
            pbBHf.Visible = false;
            pbSf.Visible = false;
            pbSt.Visible = false;

            tbPingGWpc.Clear();
            tbPingInternet.Clear();
            tbPingMain.Clear();
            tbPingNGW.Clear();
            tbTraceroute.Clear();
            lbMyIP.Text = "XXX.XXX.XXX.XXX";
            lbStatusSource.Text = null;
        }

        private void btClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmDiagnostics_Load(object sender, EventArgs e)
        {

        }

        private void btDiagStart_Click(object sender, EventArgs e)
        {
            startDiagnostic();
        }

       

        private void startDiagnostic()
        {
            clearAll();
            pnlWorking.Visible = true;
            pnlWorking.BringToFront();
            pnlWorking.Invalidate();
            this.Refresh();
            string tmpIP = conf.getMyIP();
            if (tmpIP == "" || tmpIP == null)
            {
                lbMyIP.Text = conf.myAddress;
                lbStatusSource.Text = "IP z koniguračního souboru!";
            }
            else
            {
                lbMyIP.Text = tmpIP;
                lbStatusSource.Text = "IP zjištěna online.";
            }
            lbWorking.Text = "Trasuji...";
            pnlWorking.Refresh();
            tbTraceroute.Clear();
            List<string> tmpTrac = conf.getTraceroute("8.8.8.8", false);
            foreach (string s in tmpTrac)
            {
                tbTraceroute.Text = tbTraceroute.Text + s + Environment.NewLine;
            }
            lbWorking.Text = "Dostupnost internetu...";
            pnlWorking.Refresh();
            tbPingInternet.Text = conf.Ping("8.8.8.8");
            if (!tbPingInternet.Text.Contains("Received = 0"))
            {
                pbNetT.Visible = true;
                oNet = "ANO";
            }
            else
            {
                pbNetF.Visible = true;
                oNet = "NE";
            }
            lbWorking.Text = "Dostupnost hlavního PC HKFree...";
            pnlWorking.Refresh();
            tbPingMain.Text = conf.Ping("10.107.4.100");
            if (!tbPingMain.Text.Contains("Received = 0"))
            {
                pbSt.Visible = true;
                oHKF = "ANO";
            }
            else
            {
                pbSf.Visible = true;
                oHKF = "NE";
            }
            lbWorking.Text = "Dostupnost první brány HKFree...";
            pnlWorking.Refresh();
            string firstGW = "";
            foreach (string s in tbTraceroute.Lines)
            {
                if (s.StartsWith("10.107."))
                {
                    firstGW = s;
                    break;
                }
            }
            if (firstGW == "")
            {
                tbPingNGW.Text = "Nelze zjistit první bránu v síti!";
                pbBHf.Visible = true;
                oHGW = "NE";
            }
            else
            {
                tbPingNGW.Text = conf.Ping(firstGW);
                if (!tbPingNGW.Text.Contains("Received = 0"))
                {
                    pbBHt.Visible = true;
                    oHGW = "ANO";
                }
                else
                {
                    pbBHf.Visible = true;
                    oHGW = "NE";
                }
            }
            lbWorking.Text = "Dostupnost brány PC...";
            pnlWorking.Refresh();
            string tmpPossibleGW = "";
            try
            {
                tmpPossibleGW = tbTraceroute.Lines.First().ToString();
            }
            catch (Exception)
            {
            }
            if (tmpPossibleGW != "" || tmpPossibleGW != null)
            {
                string tmpGWpc = conf.getDefaultGateway(tmpPossibleGW);
                if (tmpGWpc != null) tbPingGWpc.Text = conf.Ping(tmpGWpc);
                else tbPingGWpc.Text = "Nelze zjistit bránu počítače!";
            }
            else tbPingGWpc.Text = "Nelze zjistit bránu počítače!";
            if (!tbPingGWpc.Text.Contains("Received = 0") && tbPingGWpc.Text.Contains("Received"))
            {
                pbBPt.Visible = true;
                oPCGW = "ANO";
            }
            else
            {
                pbBPf.Visible = true;
                oPCGW = "NE";
            }
            
            if (oHKF == "ANO")
            {
                lbWorking.Text = "Test rychlosti o velikosti 1MB...";
                pnlWorking.Refresh();
                lbSpeed.Text = Speedtest.speedtest1MB(conf.speedtestAddr1M);
            }

            pnlWorking.Visible = false;
            this.Refresh();

            btSendDiag.Enabled = true;
            if (oNet == "NE" || oHGW == "NE" || oPCGW == "NE")
            {
                btFAQ.Visible = true;
            }
        }

        private void btSendDiag_Click(object sender, EventArgs e)
        {
            frmSendDiag frmSD = new frmSendDiag(conf, this);
            frmSD.Show();
        }

        private void btFAQ_Click(object sender, EventArgs e)
        {
            frmFAQ frmfq = new frmFAQ();
            frmfq.Show();
        }

        private void bt1MB_Click(object sender, EventArgs e)
        {
            try
            {
                lbWorking.Text = "Test rychlosti o velikosti 1MB...";
                pnlWorking.Visible = true;
                pnlWorking.Refresh();
                lbSpeed.Text = "";
                lbSpeed.Text = Speedtest.speedtest1MB(conf.speedtestAddr1M);
                pnlWorking.Visible = false;
                pnlWorking.Refresh();
            }
            catch (Exception)
            {
                pnlWorking.Visible = false;
                pnlWorking.Refresh();
                MessageBox.Show("Nelze provést test rychlosti!");
            }
        }

        private void bt10MB_Click(object sender, EventArgs e)
        {
            try
            {
                lbWorking.Text = "Test rychlosti o velikosti 10MB...";
                pnlWorking.Visible = true;
                pnlWorking.Refresh();
                lbSpeed.Text = "";
                lbSpeed.Text = Speedtest.speedtest10MB(conf.speedtestAddr10M);
                pnlWorking.Visible = false;
                pnlWorking.Refresh();
            }
            catch (Exception)
            {
                pnlWorking.Visible = false;
                pnlWorking.Refresh();
                MessageBox.Show("Nelze provést test rychlosti!");
            }
        }

        
    }
}
