﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;

using System.Drawing;
using System.Xml.Serialization;
using System.IO;
using System.Net.NetworkInformation;
using System.Net;
using System.Diagnostics;
using System.Reflection;
using System.Threading;
using System.Resources;
using HKFmonitor.Properties;
using System.Net.Sockets;

namespace HKFmonitor
{
    public class SysTrayApp : Form
    {
        Configuration conf;
        private System.Windows.Forms.Timer tmrZpravy;
        private System.ComponentModel.IContainer components;
        private System.Windows.Forms.Timer tmrPing;
        MessagesClass msg;
        private TCPserver os;
        Updates upd;
        TCPclient pc;
        XMLcomm xc;
        private bool newMessageAvailable;
        private Icon currentIcon = Resources.green;
        statusPing sp;
        private System.Windows.Forms.Timer tmrExit;
        internal int appState;
        private System.Windows.Forms.Timer tmrUpdates;
        private bool closeApp = false;

        [STAThread]
        public static void Main()
        {

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new SysTrayApp());


        }

        private NotifyIcon trayIcon;
        private ContextMenu trayMenu;

        public SysTrayApp()
        {
            InitializeComponent();
            newMessageAvailable = false;
            conf = new Configuration();
            sp = new statusPing();
            XmlSerializer xmls = new XmlSerializer(conf.GetType());
            try
            {
                FileStream fs = new FileStream(Application.StartupPath + "\\HKFmonitor.ini", FileMode.Open, FileAccess.Read);
                conf = (Configuration)xmls.Deserialize(fs);
                try
                {
                    fs.Close();
                }
                catch
                {
                    MessageBox.Show("Nepodařilo se načíst konfiguraci programu!");
                    firstSetup();
                }
            }
            catch
            {
                MessageBox.Show("Nepodařilo se načíst konfiguraci programu!");
                firstSetup();
            }

            if (!closeApp)
            {
                xc = new XMLcomm(conf.clientsChecksum, conf.technicsChecksume);
                checkClients();
                checkTechnics();
                loadMessages();
                startOPENserver();
                upd = new Updates();
                pc = new TCPclient(conf);

                trayMenu = new ContextMenu();
                trayMenu.MenuItems.Add("Diagnostika", showDiagnosticsForm);
                //trayMenu.MenuItems.Add("Zkušební", ukazForm);
                trayMenu.MenuItems.Add("Zprávy", showMsgForm);
                trayMenu.MenuItems.Add("Podpora", showSupportForm);
                trayMenu.MenuItems.Add("Aktualizace", showCfgUpdate);
                trayMenu.MenuItems.Add("Nastavení", showCfgForm);
                trayMenu.MenuItems.Add("Exit", OnExit);

                trayIcon = new NotifyIcon();
                trayIcon.Text = "HKFmonitor";
                trayIcon.Icon = new Icon(Resources.green, 40, 40);

                trayIcon.ContextMenu = trayMenu;
                trayIcon.BalloonTipClicked += new EventHandler(showMsgForm);
                trayIcon.Visible = true;

                appState = 2;
                tmrZpravy.Start();
                tmrPing.Start();
                if (conf.autoUpdate) tmrUpdates.Start();
            }
            else tmrExit.Start();
        }

        private void startOPENserver()
        {
            this.os = new TCPserver(conf, msg.seznamZprav);
        }

        private void firstSetup()
        {
            conf.myAddress = conf.getMyIP();
            if (conf.myAddress != null)
            {
                conf.myRoute = conf.getTraceroute("www.google.com", false);
                WebComm.fastPHPpost(conf.phpAddr, "saveMyIP", conf.myAddress);
                conf.saveConfiguration();
                firewallWorkaround();
            }
            else
            {
                MessageBox.Show("Nelze provést konfiguraci programu, zřejmě nemáte přístup na hlavní server HKfree nebo se nenacházíte v síti HKfree. Program bude nyní ukončen!");
                closeApp = true;
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            Visible = false;
            ShowInTaskbar = false;

            base.OnLoad(e);
        }

        private void OnExit(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void showDiagnosticsForm(object sender, EventArgs e)
        {
            frmDiagnostics msgDg = new frmDiagnostics(conf);
            msgDg.Show();
        }

        private void ukazForm(object sender, EventArgs e)
        {
            Form1 form1 = new Form1(conf);
            form1.Show();
        }

        private void showMsgForm(object sender, EventArgs e)
        {
            frmMessages msgForm = new frmMessages(msg);
            msgForm.Shown += new EventHandler(msgForm_Shown);
            msgForm.Show();
        }

        private void msgForm_Shown(Object sender, EventArgs e)
        {
            newMessageAvailable = false;
            changeIcon();
        }

        private void showSupportForm(object sender, EventArgs e)
        {
            frmSupport frmSupport1 = new frmSupport(conf);
            frmSupport1.Show();
        }

        private void showCfgForm(object sender, EventArgs e)
        {
            frmCfg frmCfg1 = new frmCfg(conf);
            frmCfg1.Show();
        }

        private void showCfgUpdate(object sender, EventArgs e)
        {
            frmUpdate frmUpd1 = new frmUpdate(conf);
            if (frmUpd1.ShowDialog() == DialogResult.OK)
            {
                if (conf.autoUpdate) tmrUpdates.Start();
                if (!conf.autoUpdate) tmrUpdates.Stop();
            }
        }

        protected override void Dispose(bool isDisposing)
        {
            if (isDisposing)
            {
                try
                {
                    trayIcon.Dispose();
                }
                catch (Exception)
                {

                }
            }

            base.Dispose(isDisposing);
        }

        internal void checkClients()
        {
            List<string> tmpClients = new List<string>();
            var cli = xc.clientsXMLparser(WebComm.fastPHPpost(conf.phpAddr, "clients", conf.clientsChecksum.ToString()));
            if (cli.Count() > 0)
            {
                conf.clientsChecksum = xc.clchecksum;
                conf.clients = cli;
                conf.saveConfiguration();
            }
        }

        internal void checkMessages()
        {
            List<MessageStruct> zpr;
            if (appState == 0 || File.Exists(conf.appStart + "\\Debug"))
            {
                foreach (string cl in conf.clients)
                {
                    zpr = xc.messagesXMLparser(pc.getListSync("$message;" + msg.getLatest(), cl));
                    addMessages(zpr);
                }
            }
            else
            {
                zpr = xc.messagesXMLparser(WebComm.fastPHPpost(conf.phpAddr, "messages", msg.getLatest().ToString()));
                addMessages(zpr);
            }
        }

        internal void checkTechnics()
        {
            List<string> tmpClients = new List<string>();
            var tech = xc.technicsXMLparser(WebComm.fastPHPpost(conf.phpAddr, "technics", conf.technicsChecksume.ToString()));
            if (tech.Count() > 0)
            {
                conf.technicsChecksume = xc.tchecksume;
                conf.technics = tech;
                conf.saveConfiguration();
            }
        }

        internal void addMessages(List<MessageStruct> zpr)
        {
            bool saveMsg = false;
            if (zpr == null) zpr = new List<MessageStruct>();
            foreach (MessageStruct ms in zpr)
            {
                if (!msg.seznamZprav.Any(prod => prod.ID == ms.ID))
                {
                    msg.seznamZprav.Add(ms);
                    saveMsg = true;
                    newMessageAvailable = true;
                    changeIcon();
                    string tmpText = ms.text;
                    if (tmpText == "" || tmpText == null) tmpText = " ";
                    trayIcon.ShowBalloonTip(3000, "Nová zpráva číslo " + ms.ID + " ze dne " + conf.tryParseDate(ms.datum).ToString("dd/M/yyyy"), tmpText, ToolTipIcon.Info);
                }
            }
            if (saveMsg == true) msg.saveMessages();
        }

        internal void loadMessages()
        {
            msg = new MessagesClass();
            XmlSerializer xmls = new XmlSerializer(msg.GetType());
            try
            {
                FileStream fs = new FileStream(Application.StartupPath + "\\HKFmonitor.msg", FileMode.Open, FileAccess.Read);
                msg = (MessagesClass)xmls.Deserialize(fs);
                try
                {
                    fs.Close();
                }
                catch
                {
                    MessageBox.Show("Nepodařilo se načíst zprávy!");
                }
            }
            catch
            {
                MessageBox.Show("Nepodařilo se načíst zprávy!");
            }
        }

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.tmrZpravy = new System.Windows.Forms.Timer(this.components);
            this.tmrPing = new System.Windows.Forms.Timer(this.components);
            this.tmrExit = new System.Windows.Forms.Timer(this.components);
            this.tmrUpdates = new System.Windows.Forms.Timer(this.components);
            this.SuspendLayout();
            // 
            // tmrZpravy
            // 
            this.tmrZpravy.Interval = 10000;
            this.tmrZpravy.Tick += new System.EventHandler(this.tmrZpravy_Tick);
            // 
            // tmrPing
            // 
            this.tmrPing.Interval = 5000;
            this.tmrPing.Tick += new System.EventHandler(this.tmrPing_Tick);
            // 
            // tmrExit
            // 
            this.tmrExit.Tick += new System.EventHandler(this.tmrExit_Tick);
            // 
            // tmrUpdates
            // 
            this.tmrUpdates.Interval = 5000;
            this.tmrUpdates.Tick += new System.EventHandler(this.tmrUpdates_Tick);
            // 
            // SysTrayApp
            // 
            this.ClientSize = new System.Drawing.Size(284, 261);
            this.Name = "SysTrayApp";
            this.Load += new System.EventHandler(this.SysTrayApp_Load);
            this.ResumeLayout(false);

        }

        private void tmrZpravy_Tick(object sender, EventArgs e)
        {

            tmrZpravy.Stop();
            tmrZpravy.Interval = conf.intMessages;
            Thread t = new Thread(new ThreadStart(checkMessages));
            t.IsBackground = true;
            t.Start();
            tmrZpravy.Start();
        }

        private void tmrPing_Tick(object sender, EventArgs e)
        {
            tmrPing.Stop();
            tmrPing.Interval = conf.intOnline;
            sp.lastCheckIP = conf.checkAddress;
            sp.ipOut = conf.checkAddress;
            sp.ipIn = conf.checkGWaddress;
            sp.ipOutFail = 0;
            sp.ipInFail = 0;
            sp.good = false;
            sp.initial = true;
            statusPing(sp);
            tmrPing.Start();
        }

        private void changeIcon()
        {
            Icon icon;
            if (newMessageAvailable) icon = Resources.message;
            else if (appState == 2) icon = Resources.green;
            else if (appState == 1) icon = Resources.yellow;
            else icon = Resources.red;
            
            if (currentIcon != icon)
            {
                currentIcon = icon;
                trayIcon.Icon = new Icon(currentIcon, 40, 40);
            }
        }

        //-----------------PING--------------------------

        public static bool ping5(string nameOrAddress)
        {
            bool stav = false;
            for (int i = 0; i < 11; i++)
            {
                ping(nameOrAddress);
                if (stav == true) i = 11;
                if (stav == false) System.Threading.Thread.Sleep(1000);
            }
            return stav;
        }

        private void statusPing(statusPing sp)
        {
            if (sp.initial == true)
            {
                sp.lastCheckIP = conf.checkAddress;
                pingAsync(conf.checkAddress);
            }
            if (sp.initial == false)
            {
                if (sp.good == true)
                {
                    if (sp.lastCheckIP == conf.checkAddress)
                    {
                        appState = 2;
                        changeIcon();
                    }
                    if (sp.lastCheckIP == conf.checkGWaddress)
                    {
                        appState = 1;
                        changeIcon();
                    }
                }
                if (sp.good == false)
                {
                    if (sp.lastCheckIP == conf.checkAddress && sp.ipOutFail < 5)
                    {
                        sp.ipOutFail++;
                        pingAsync(conf.checkAddress);
                    }
                    else if (sp.lastCheckIP == conf.checkAddress && sp.ipOutFail == 6)
                    {
                        sp.lastCheckIP = conf.checkGWaddress;
                        pingAsync(conf.checkGWaddress);
                    }
                    else if (sp.lastCheckIP == conf.checkGWaddress && sp.ipInFail < 6)
                    {
                        sp.ipInFail++;
                        pingAsync(conf.checkGWaddress);
                    }
                    else
                    {
                        appState = 0;
                        changeIcon();
                    }
                }
            }
        }

        private void pingAsync(string nameOrAddress)
        {
            Ping p = new Ping();
            p.PingCompleted += p_PingAsyncCompleted;
            p.SendAsync(nameOrAddress, 1500, nameOrAddress);
        }

        private void p_PingAsyncCompleted(object sender, PingCompletedEventArgs e)
        {
            if (e.Reply.Status != IPStatus.Success)
            {
                sp.initial = false;
                sp.good = false;
                statusPing(sp);
            }
            else if (e.Reply.Status == IPStatus.Success)
            {
                sp.initial = false;
                sp.good = true;
                statusPing(sp);
            }
        }

        public static bool ping(string nameOrAddress)
        {
            bool pingable = false;
            Ping pinger = new Ping();

            try
            {
                PingReply reply = pinger.Send(nameOrAddress);

                pingable = reply.Status == IPStatus.Success;
            }
            catch (PingException)
            {
            }
            return pingable;
        }

        private void SysTrayApp_Load(object sender, EventArgs e)
        {
        }

        private void tmrExit_Tick(object sender, EventArgs e)
        {
            Close();
        }

        private void autoUpdate()
        {
            string newVer = upd.checkVersion(conf.updateAddr, conf.appversion);
            if (newVer != null)
            {
                upd.performUpdate(true, conf.updateAddr);
            }
        }

        private void tmrUpdates_Tick(object sender, EventArgs e)
        {
            tmrUpdates.Stop();
            tmrUpdates.Interval = conf.intUpdates;
            Thread t = new Thread(new ThreadStart(autoUpdate));
            t.IsBackground = true;
            t.Start();
            tmrUpdates.Start();
        }

        private void firewallWorkaround()
        {
            IPAddress ipAddress = Dns.GetHostEntry(Dns.GetHostName()).AddressList[0];
            IPEndPoint ipLocalEndPoint = new IPEndPoint(ipAddress, 12345);

            TcpListener t = new TcpListener(ipLocalEndPoint);
            t.Start();
            t.Stop();
        }

        public EventHandler msgForm1_Shown { get; set; }
    }
}
