﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Collections.Generic;

namespace HKFmonitor
{
    internal class TCPserver
    {
        public class StateObject
        {
            public Socket workSocket = null;
            public const int BufferSize = 262144;
            public bool isScanner = false;
            public string scannerAddress = "";
            public byte[] buffer = new byte[BufferSize];
            public StringBuilder sb = new StringBuilder();
        }

        internal List<StateObject> sos;
        private Configuration conf;
        private List<MessageStruct> msg;
        public string NetIP;
        public int NetPort;
        private TcpListener tcl;

        public event AsyncCallback connCame;

        public event AsyncCallback dataRecd;

        public event AsyncCallback dataWritten;

        public event AsyncCallback fileWritten;

        public TCPserver(Configuration confI, List<MessageStruct> msgI)
        {
            sos = new List<StateObject>();

            try
            {
                this.conf = confI;
                this.msg = msgI;
                this.NetIP = this.conf.openIP;
                this.NetPort = this.conf.openPort;
                this.connCame = (AsyncCallback)Delegate.Combine(this.connCame, new AsyncCallback(this.OPENserver_connCame));
                this.dataRecd = (AsyncCallback)Delegate.Combine(this.dataRecd, new AsyncCallback(this.OPENserver_dataRecd));
                this.dataWritten = (AsyncCallback)Delegate.Combine(this.dataWritten, new AsyncCallback(this.OPENserver_dataWritten));
                this.fileWritten = (AsyncCallback)Delegate.Combine(this.fileWritten, new AsyncCallback(this.OPENserver_fileWritten));
                this.tcl = new TcpListener(this.NetIP == "*" ? IPAddress.Any : IPAddress.Parse(this.NetIP), this.NetPort);
                tcl.ExclusiveAddressUse = false;

                this.tcl.Start();
                this.tcl.BeginAcceptSocket(this.connCame, this.tcl);
            }
            catch { }
        }

        private void dataRead(StateObject so)
        {
            string data = so.sb.ToString();
            so.buffer = new byte[262144];
            if (data[0] == '$')
            {
                if (data.StartsWith("$message"))
                {
                    string outputStart = "<?xml version=\"1.0\" encoding=\"WINDOWS-1250\"?><LIST version=\"3.2.1\">";
                    string outputEnd = "</LIST>";
                    data = data.Substring(data.IndexOf(';') + 1);
                    string list ="";
                    foreach (MessageStruct ms in msg)
                    {
                        if (ms.ID > conf.tryParseInt(data)) list = list + "<ITEM ID=\"" + ms.ID + "\" SUBNET=\"" + s(ms.subnet) + "\" DATE=\"" + ms.datum + "\" PARRENT=\"" + s(ms.parrent) + "\" TEXT=\"" + s(ms.text) + "\" />";
                    }
                    string output;
                    if (list != "")
                    {
                        output = outputStart + list + outputEnd;
                    }
                    else output = "empty";
                    this.writeData(so, Encoding.GetEncoding(0x4e2).GetBytes(output + ";"));
                    return;
                }
            }
        }

        private void OPENserver_connCame(IAsyncResult ar)
        {
            try
            {
                StateObject so = new StateObject();

                List<StateObject> soTMP = new List<StateObject>();
                foreach (StateObject sot in sos)
                {
                    if (sot.workSocket != null && (sot.workSocket.Connected || so.isScanner))
                    {


                        soTMP.Add(sot);
                    }
                }
                sos = soTMP;
                sos.Add(so);
                so.workSocket = this.tcl.EndAcceptSocket(ar);
                this.tcl.BeginAcceptSocket(this.connCame, this.tcl);
                so.workSocket.BeginReceive(so.buffer, 0, 262144, SocketFlags.None, this.dataRecd, so);
            }
            catch (Exception)
            {
                this.tcl.BeginAcceptSocket(this.connCame, this.tcl);
            }
        }

        private void OPENserver_dataRecd(IAsyncResult ar)
        {
            try
            {
                Thread.Sleep(5);
                StateObject so = ((StateObject)ar.AsyncState);
                int count = so.workSocket.EndReceive(ar);
                if (count == 0)
                    return;
                string str = Encoding.GetEncoding(0x4e2).GetString(so.buffer, 0, count).Replace("\r", "");
                so.workSocket.BeginReceive(so.buffer, 0, 262144, SocketFlags.None, this.dataRecd, so);
                so.sb.Append(str);
                if (so.workSocket.Available == 0)
                {

                    this.dataRead(so);
                    if (!so.isScanner)
                        so.workSocket.Disconnect(false);

                }
                else
                {
                    so.workSocket.BeginReceive(so.buffer, 0, 262144, SocketFlags.None, this.dataRecd, so);
                }

            }
            catch (Exception)
            {
                this.tcl.BeginAcceptSocket(this.connCame, this.tcl);
            }
        }

        private void OPENserver_dataWritten(IAsyncResult ar)
        {
            ((StateObject)ar.AsyncState).workSocket.EndSend(ar);
        }

        private void OPENserver_fileWritten(IAsyncResult ar)
        {
            ((StateObject)ar.AsyncState).workSocket.EndSend(ar);
        }

        public void writeData(StateObject so, byte[] dtToSend)
        {
            so.workSocket.BeginSend(dtToSend, 0, dtToSend.Length, SocketFlags.None, this.dataWritten, so);
        }

        internal string s(string t)
        {
            return System.Security.SecurityElement.Escape(t);
        }

    }
}
